#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$exicutable agr_file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program combines .agr files into a single file.
    h - this message
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("o:O:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $set;
my @sets;
while (<>) {
    if (/\@\s+s(\d+)\s+legend\s+\"([^\"]+)\"/) {
	my ($s, $legend) = ($1, $2);
	if (defined $set) {
	    push @sets, $set;
	}
	$set = {
	    "legend" => $legend,
	    "points" => []
	    };
    } else {
	if (/^\d/) {
	    push @{$set->{points}}, $_;
	}
    }
}
push @sets, $set;


print<<EOP;
@    world xmin 1980
@    world xmax 2010
@    world ymin 0
@    world ymax 100
EOP

my $s = 0;
my $color = 0;
my $last_name;
my $pattern_index;
my @patterns = (1,4,2);
foreach my $set (sort {$a->{legend}<=>$b->{legend}} @sets) {
    my $legend = $set->{legend};
    next unless ($legend =~ /most specific/);

    print "@ s$s legend \"$legend\"\n";
    my ($name) = split /\-/, $legend;
    if ($last_name ne $name) {
	$pattern_index = 0;
	$color++;
	$color++ if ($color == 5);
    } else {
	$pattern_index++;
    }

    print "@ s$s color $color\n";
    my $pattern = $patterns[$pattern_index];
    print "@ s$s line pattern $pattern\n";

    $last_name = $name;
    $s++;
    foreach my $point (@{$set->{points}}) {
	print $point;
    }
}
