#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;

my $xmin = 2003.5;
my $xmax = 2009.5;
my $ymax = 255;

my %legend2end;
my %legend2diff;
my ($legend,$s);
while (<>) {
    if (/world xmin/) {
	print "@  world xmin $xmin\n";
    } elsif (/world xmax/) {
	print "@  world xmax $xmax\n";
    } elsif (/world ymax/) {
	print "@  world ymax $ymax\n";

    } elsif (/@    xaxis  tick major/) {
	print "@    xaxis  tick major 1\n";
    } elsif (/@\s+xaxis\s+label\s+\"/) {
	print "@    xaxis  label \"\"\n";

    } elsif (/@    legend/) { 
	print "@    legend .8, .5\n";
    } else {
	print;
    }

    if (/@\s+s(\d+)\s+legend\s+"([^\"]+)/) {
	($s,$legend) = ($1,$2);
    } elsif (/\s*([\d\.]+)\s+([\d\.]+)/) {
	my ($x, $y) = ($1, $2);
	if ($legend =~ /IETF/) {
	    my $diff = 255 - $y;
	    my $legend_diff = $legend2diff{$legend};
	    if (!defined $legend_diff && $diff < 0) {
		$legend2end{$legend} = $x;
		$legend2diff{$legend} = $diff;
	    }
	}
    }
}
$s++;
PrintLines($s);
PrintNewWorld();

sub PrintLines {
    my $s = $_[0];
    foreach my $legend (keys %legend2end) {
	my $end = $legend2end{$legend};
print<<EOP;
@    s$s hidden false
@    s$s type xy
@    s$s symbol 0
@    s$s symbol size 1.000000
@    s$s symbol color 1
@    s$s symbol pattern 1
@    s$s symbol fill color 1
@    s$s symbol fill pattern 0
@    s$s symbol linewidth 1.0
@    s$s symbol linestyle 1
@    s$s symbol char 65
@    s$s symbol char font 0
@    s$s symbol skip 0
@    s$s line type 3
@    s$s line linestyle 3
@    s$s line linewidth 1.0
@    s$s line color 1
@    s$s line pattern 1
@    s$s baseline type 0
@    s$s baseline off
@    s$s dropline off
@    s$s fill type 2
@    s$s fill rule 0
@    s$s fill color 1
@    s$s fill pattern 1
@    s$s avalue off
@    s$s avalue type 2
@    s$s avalue char size 1.000000
@    s$s avalue font 0
@    s$s avalue color 1
@    s$s avalue rot 0
@    s$s avalue format general
@    s$s avalue prec 3
@    s$s avalue prepend ""
@    s$s avalue append ""
@    s$s avalue offset 0.000000 , 0.000000
@    s$s errorbar on
@    s$s errorbar place both
@    s$s errorbar color 1
@    s$s errorbar pattern 1
@    s$s errorbar size 1.000000
@    s$s errorbar linewidth 1.0
@    s$s errorbar linestyle 1
@    s$s errorbar riser linewidth 1.0
@    s$s errorbar riser linestyle 1
@    s$s errorbar riser clip off
@    s$s errorbar riser clip length 0.100000
@    s$s legend  ""
EOP
	print "$end 0\n";
	print "$end 255\n";
	$s++;
    }
}

sub PrintNewWorld() {
    my @ticks;
    my $lmax;
    foreach my $legend (sort {$legend2end{$a}<=>$legend2end{$b}}
	keys %legend2end) {
print STDERR "legend:$legend\n";
	my $end = $legend2end{$legend};
	my $name = $legend;
	my ($year, $mon) = ($end =~ /([^\.]+)(\.\d+){0,1}/);
	$mon = sprintf ("\%d", ($mon*12));
	$mon = 11 if ($mon >= 12);
	$mon = "0".$mon if ($mon < 10);

	$legend =~ s/IETF\s*//g;
	($legend) = split /\s+/,$legend;
	my $label = "$legend $year-$mon";

	push @ticks, [$end, $label];
	my $len = length($label);
	if ($len > $lmax) {
	    $lmax = $len;
	}
    }

    my $num_ticks=@ticks;
    my $tick_text;
    foreach my $index (0..$#ticks) {
	my ($end, $label) = @{$ticks[$index]};
	$label = sprintf("%".$lmax."s", $label);
	$tick_text .= "@    xaxis  tick major $index, $end\n";
	$tick_text .= "@    xaxis  ticklabel  $index, \"$label\"\n";
    }
#    my $label_x_offset = -0.01*($lmax/2);
    my $label_x_offset = -0.065;
print<<EOP;
\@g1 on
\@g1 hidden false
\@g1 type XY
\@g1 stacked false
\@g1 bar hgap 0.000000
\@g1 fixedpoint off
\@g1 fixedpoint type 0
\@g1 fixedpoint xy 0.000000, 0.000000
\@g1 fixedpoint format general general
\@g1 fixedpoint prec 6, 6
\@with g1
@    world xmin $xmin
@    world xmax $xmax
@    world ymin 0
@    world ymax 1
@    stack world 0, 0, 0, 0
@    znorm 1
@    view xmin 0.150000
@    view xmax 1.150000
@    view ymin 0.150000
@    view ymax 0.850000
@    title ""
@    title font 0
@    title size 1.500000
@    title color 1
@    subtitle ""
@    subtitle font 0
@    subtitle size 1.000000
@    subtitle color 1
@    xaxes scale Normal
@    yaxes scale Normal
@    xaxes invert off
@    yaxes invert off
@    xaxis  on
@    xaxis  type zero false
@    xaxis  offset 0.000000 , 0.000000
@    xaxis  bar on
@    xaxis  bar color 1
@    xaxis  bar linestyle 1
@    xaxis  bar linewidth 1.0
@    xaxis  label ""
@    xaxis  label layout para
@    xaxis  label place auto
@    xaxis  label char size 1.000000
@    xaxis  label font 0
@    xaxis  label color 1
@    xaxis  label place normal
@    xaxis  tick on
@    xaxis  tick major 2000
@    xaxis  tick minor ticks 1
@    xaxis  tick default 6
@    xaxis  tick place rounded true
@    xaxis  tick in
@    xaxis  tick major size 0.000000
@    xaxis  tick major color 1
@    xaxis  tick major linewidth 1.0
@    xaxis  tick major linestyle 1
@    xaxis  tick major grid off
@    xaxis  tick minor color 1
@    xaxis  tick minor linewidth 1.0
@    xaxis  tick minor linestyle 1
@    xaxis  tick minor grid off
@    xaxis  tick minor size 0.500000
@    xaxis  ticklabel on
@    xaxis  ticklabel format general
@    xaxis  ticklabel prec 5
@    xaxis  ticklabel formula ""
@    xaxis  ticklabel append ""
@    xaxis  ticklabel prepend ""
@    xaxis  ticklabel angle 25
@    xaxis  ticklabel skip 0
@    xaxis  ticklabel stagger 0
@    xaxis  ticklabel place normal
@    xaxis  ticklabel offset spec
@    xaxis  ticklabel offset $label_x_offset , 0.030000
@    xaxis  ticklabel start type auto
@    xaxis  ticklabel start 0.000000
@    xaxis  ticklabel stop type auto
@    xaxis  ticklabel stop 0.000000
@    xaxis  ticklabel char size 0.800000
@    xaxis  ticklabel font 0
@    xaxis  ticklabel color 1
@    xaxis  tick place both
@    xaxis  tick spec type both
@    xaxis  tick spec $num_ticks
$tick_text
@    yaxis  on
@    yaxis  type zero false
@    yaxis  offset 0.000000 , 0.000000
@    yaxis  bar on
@    yaxis  bar color 1
@    yaxis  bar linestyle 1
@    yaxis  bar linewidth 1.0
@    yaxis  label ""
@    yaxis  label layout para
@    yaxis  label place auto
@    yaxis  label char size 1.000000
@    yaxis  label font 0
@    yaxis  label color 1
@    yaxis  label place normal
@    yaxis  tick on
@    yaxis  tick major 2000
@    yaxis  tick minor ticks 1
@    yaxis  tick default 6
@    yaxis  tick place rounded true
@    yaxis  tick in
@    yaxis  tick major size 1.000000
@    yaxis  tick major color 1
@    yaxis  tick major linewidth 1.0
@    yaxis  tick major linestyle 1
@    yaxis  tick major grid off
@    yaxis  tick minor color 1
@    yaxis  tick minor linewidth 1.0
@    yaxis  tick minor linestyle 1
@    yaxis  tick minor grid off
@    yaxis  tick minor size 0.500000
@    yaxis  ticklabel on
@    yaxis  ticklabel format general
@    yaxis  ticklabel prec 5
@    yaxis  ticklabel formula ""
@    yaxis  ticklabel append ""
@    yaxis  ticklabel prepend ""
@    yaxis  ticklabel angle 0
@    yaxis  ticklabel skip 0
@    yaxis  ticklabel stagger 0
@    yaxis  ticklabel place normal
@    yaxis  ticklabel offset auto
@    yaxis  ticklabel offset 0.000000 , 0.010000
@    yaxis  ticklabel start type auto
@    yaxis  ticklabel start 0.000000
@    yaxis  ticklabel stop type auto
@    yaxis  ticklabel stop 0.000000
@    yaxis  ticklabel char size 1.000000
@    yaxis  ticklabel font 0
@    yaxis  ticklabel color 1
@    yaxis  tick place both
@    yaxis  tick spec type none
@    altxaxis  off
@    altyaxis  off
@    legend on
@    legend loctype view
@    legend 0.5, 0.8
@    legend box color 1
@    legend box pattern 1
@    legend box linewidth 1.0
@    legend box linestyle 1
@    legend box fill color 0
@    legend box fill pattern 1
@    legend font 0
@    legend char size 1.000000
@    legend color 1
@    legend length 4
@    legend vgap 1
@    legend hgap 1
@    legend invert false
@    frame type 0
@    frame linestyle 1
@    frame linewidth 1.0
@    frame color 1
@    frame pattern 1
@    frame background color 0
@    frame background pattern 0
EOP
}
