#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use Getopt::Std;
use Time::Local;
use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$0 [-hl] events\n";
sub Help() {
    print <<EOP;
$usage
    h - this message
    l - print this as lines, not stacked
EOP
}

my %opts;
unless (getopts("lh",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $OUTPUT_TYPE = "stacks";
if (defined $opts{l}) {
    $OUTPUT_TYPE = "lines";
}


my %opts;

my %time;
my %org;
my %orgTotal;
my %total;
ParseEvents();
#PrintTotalLine();
PrintStacks();

sub ParseEvents {
    while (<>) {
	s/
(\n){0,1}//g;
	my ($block, $date, $type, $org, $name) = split /\s*\|\s*/;
	my $time = Date2Time($date);
	if ($type ne "reserved") {
	    my ($length) = ($block =~ /\/(\d+)/);
	    my $size;
	    if ($length == 8) {
		$size = 1;
	    } elsif ($length < 8) {
		$size = 1<<(8-$length);
	    } else {
		$size = 1/(1<<($length-8));
	    }
	    if ($type eq "returned") {
		$size *= -1;
	    }
	    Add($org, $time, $size);
	}
    }
}

sub Add {
    my ($org, $time, $size) = @_;
    foreach my $o ($org) {
	$time{$time}{$o} += $size;
	$org{$o}{$time} += $size;
	$orgTotal{$o} += $size;
    }	
    unless ($org =~ /IETF/) {
	$total{$time} += $size;
    }
}

sub PrintTotalLine {
    my $total = 0;
    foreach my $time (sort {$a<=>$b} keys %total) {
	$total += $total{$time};
	print "$time $total\n";
    }
}

sub PrintLines {
    my $s = 0;
    my @orgs = reverse sort {$orgTotal{$a}<=>$orgTotal{$b}} keys %orgTotal;
    foreach my $org (@orgs) {
	my $name = $org;
	unless ($org eq "IETF") {
	    $name =~ s/IEFT//;
	}
	print "\@  s$s   legend \"$name\"\n";
	$s++;
	my $total;
	foreach my $time (sort {$a<=>$b} keys %{$org{$org}}) {
	    my $value = $org{$org}{$time};
	    $total += $value;
	    print "$time $total\n";
	}
    }
}

sub PrintStacks {
    my @orgs = reverse sort {
	my $a_IETF = (($a =~ /IETF/) || ($a =~ /ulticast/));
	my $b_IETF = (($b =~ /IETF/) || ($a =~ /ulticast/));
	if ($a_IETF) {
	    if ($b_IETF) {
		if ($a eq "IETF") {
		    return 1;
		} else {
		    return -1;
		}
	    }
	    return -1;
	} elsif ($b_IETF) {
	    return 1;
	}
	$orgTotal{$a}<=>$orgTotal{$b}
	} keys %orgTotal;
    my @times = sort {$a<=>$b} keys %time;
    push @times, Date2Time(Time2Date(time));
    my $s = 0;
    my %sums;
    my %values;
    foreach my $time (@times) {
	my $total;
	foreach my $org (@orgs) {
	    if ($OUTPUT_TYPE eq "lines") {
		$total = 0;
	    }
	    my $value = $org{$org}{$time};
	    $total += $value;
	    $sums{$org} += $total;
	    $values{$time}{$org} = $sums{$org};
	}
    }
    my $s = 0;
    foreach my $org (reverse @orgs) {
	print "\@  s$s   legend \"$org\"\n";
	$s++;
	foreach my $time (@times) {
	    my $value = $values{$time}{$org};
	    print "$time $value\n";
	}
    }
}

sub Date2Time {
    my ($date) = @_;
    my ($year,$mon,$day) = ($date =~ /(\d{4})(\d\d)(\d\d)/);
    my ($hour, $min, $sec) = ( 0,0,0 );
    return $year + ($mon-1)/12;
    return Time::Local::timelocal($sec,$min,$hour,$day,$mon-1,$year);
}


sub Time2Date {
    my ($time) = @_;
    unless (defined $time) {
	return;
    }
    my ($sec, $min, $hour, $day, $mon, $year) = localtime($time);
    $day = "0".$day if ($day < 10);
    $mon++;
    $mon = "0".$mon if ($mon < 10);
    $year += 1900;
    return "$year$mon$day";
}
