#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use Time::Local;
use strict;

# This parses the list of IPv6 global unicast assignments made to the RIRs
# by IANA on http://www.iana.org/assignments/ipv6-unicast-address-assignments

my %month = BuildMonth();
my $data;
my @events;
Parse();
#AddRfc();
PrintEvents();

sub Parse {
    while (<>) {
      last if /^Global Unicast Prefix Assignment/;
    }

    while (<>) {
        next if /^-/;
	next if /^\s*$/;
	last if /^Notes/;

	s/RIPE NCC/RIPE/;
	my ($block, $org, $day, $month, $year, @rest) = split /\s+/;
	my $time = GetTime($year, $month, $day);

	my $name = $org;
	my $type = "allocated";

	if ($block eq "2001:0000::/23") {
	  $name = "IANA experimental";
	  $type = "special-use";
	}
	elsif ($block eq "2001:0C00::/23") {
#	  $org = "IANA";
	  $name = "critical infrastructure, experimental allocations, and non-routable documentation (2001:0DB8::/32)";
	  $type = "special-use";
	}
	elsif ($org eq "6to4") {
	  $org = "IANA";
	  $type = "special-use";
	}
	elsif ($org eq "6BONE") {
	  $org = "IANA";
	  $type = "special-use";
	}
	elsif ($org eq "RESERVED" && $block eq "2001:3C00::/22") {
	  $org = "IANA";
	  $name = "reserved for RIPE";
	  $type = "reserved";
	}

	AddEvent($block, $time, $type, $org, $name);
    }
}

sub AddEvent {
    my ($block, $date, $type, $org, $name) = @_;
    push @events, {
	"block" => $block,
	"date" => $date,
	"type" => $type,
	"org" => $org,
	"name" => $name
	};
}

sub PrintEvents {
    foreach my $event ( sort {
	    if ($a->{date} == $b->{date}) {
		return $a->{block}<=>$b->{block}
	    }
	    $a->{date}<=>$b->{date}
	    } @events) {
	my ($block, $date, $type, $org, $name) = (
	    $event->{"block"},$event->{"date"}, $event->{"type"},
	    $event->{"org"}, $event->{"name"});
	printf "%-19s| %-9s| %-10s| %-10s| %s\n", 
	    $block, $date, $type, $org, $name;
    }
}

sub BuildMonth {
    my %month;
    my $index = 1;
    foreach my $mon (qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec)) {
	$month{$mon} = $index++;
    }
    return %month;
}

sub GetTime {
    my ($year, $month, $day) = @_;
    if ($year < 50) {
	$year += 2000;
    } else {
	$year += 1900;
    }
    return sprintf("%04d%02d%02d", $year, $month{$month}, $day);
}
