#!  /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = 
"usage:$0 [-n num-lines-print] [-p pattern] [-s skip-lines] file(s)\n";
sub Help() {
    print <<EOP;
$usage
    This program skips to <pattern> in the input file and prints
    from that point on.
    h - this message

    m - skip to first instance of patter
    s - skip the first skip-lines

    p - print only print-lines
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("n:p:s:h",\%opts)) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $pattern = $opts{p};
my $print_lines = $opts{n};
my $skip_lines = $opts{s};

# 7434678:530000 org:            ORG-CI9-RIPE

my $linenum = 0;
while (<>) {
    $linenum++;
    last if (!defined $skip_lines || $linenum >  $skip_lines);
}

unless (/$pattern/) {
    while(<>) {
	$linenum++;
	last if (/$pattern/);
    }
}

my $printing = $linenum;
print "# linenum:$linenum\n";
print;
while(<>) { 
    $linenum++;
    last if (defined $print_lines && $linenum-$printing > $print_lines);
    print;
    last unless (/[^\s]/);
}

