#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$0 [-h] -s source whois.dump\n";
sub Help() {
    print <<EOP;
$usage
    This program prints to STDOUT the dump it reads
    and adds source:[source].
    h - this message
    s - source LACNIC, RIPE, etc
EOP
}

use Getopt::Std;
my %opts;
unless (getopts("s:h",\%opts) && defined $opts{s}) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}
my $source = $opts{s};

my $current;
my %arg2value;
while(<>) {
    if (/^\s*$/) {
	if (defined $current && !defined $arg2value{source}) {
	    print "source:     $source\n";
	}
	undef $current;
    } elsif (/^([^:]+):\s+(.+)/) {
	my ($arg, $value) = ($1, $2);
	unless (defined $current) {
	    $current = $arg;
	}
	$arg =~ y/A-Z/a-z/;
	$arg2value{$arg} = $value;
    }
    print;
}
