#! /usr/local/bin/perl
## 
## whois IPv4 data analysis
## Copyright (C) 2006 The Regents of the University of California.
## 
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
## 
## written by Bradley Huffaker <bradley@caida.org>
## 
## 

#use lib '/usr/local/pkg/caida_coral-3.6.0/Coral/lib/';
#use CAIDA::ASFinder;
use Getopt::Std;
use Time::Local;
use Socket;
use strict;
my ($exicutable) = ($0 =~ /([^\/]+)$/);
my $usage = "usage:$0 [-h] [-r rir] -i iana.events inet{6}num\n";
sub Help() {
    print <<EOP;
$usage
    h - this message
    r - rir name
EOP
}

my %opts;
unless (getopts("r:i:lh",\%opts) && defined $opts{i}) {
    print STDERR $usage;
    exit -1;
}
if (defined $opts{h}) {
    Help();
    exit(0);
}

my $rir_target = $opts{r};

my %sizes; # $sizes{rir}{date}{sizes} = size_frequency
my %iana2rir;
my %len2size = BuildLengthSize();
my %len2mask = BuildLengthMask(%len2size);
if (defined $opts{i}) {
    %iana2rir = ParseIana($opts{i});
}

my %prefix;
my @info;
ParseWhoisEvents();
CountRirs();
PrintRirs();

sub ParseIana {
    my ($file) = @_;

    my %iana2rir;
    open (IN, "<$file") || die("Unable to open '$file':$!");
    my $linenum;
    while (<IN>) {
	$linenum++;
	s/#.*//;
	next unless (/[^\s]/);
	my ($prefix_len, $date, $type, $org, $name) = split /\s*\|\s*/;
	if ($prefix_len =~ /([\d\.]+)\/(\d+)/) {
	    my ($prefix, $len) = ($1, $2);
	    if ($len == 8 && $prefix =~ /^(\d+)/) {
		my $front = $1;
		$iana2rir{$front} = $org;
		$date = DateFraction($date);
		$sizes{$org}{"IANA"}{$date}{$len2size{8}}++;
	    }
	} else {
	    print STDERR "$file\[$linenum\] failed to parse $_";
	}
    }
    close IN;
    return %iana2rir;
}

sub ParseWhoisEvents {
    my $linenum = 0;
my $last_rir;
my $key =0;
my %len;
    while (<>) {
	my @values = split /\t/;
	foreach my $value (@values) { $value =~ s/^\s+//; $value=~ s/\s+$//;}
	my ($family, $prefixes, $org, $country, $type, $date, $rir) = @values;
	$rir =~ s/#.*//;
	$rir =~ y/a-z/A-Z/;
	$rir =~ s/^\s+//;
	$rir =~ s/\s+$//;
#next if ($rir =~ /APNIC/);
#next if ($rir =~ /ARIN/);
#next if ($rir =~ /RIPE/);

	next if ($org =~ /^IANA-/);
	next if ($org =~ /^AFRINIC-/);
	next if ($org =~ /^APNIC-/);
	next if ($org =~ /^ARIN-/);
	next if ($org =~ /^LACNIC-/);
	next if ($org =~ /^RIPE-/);
	next if ($org =~ /^ERX-/);
	next if ($org =~ /^NET\d+$/);
	next if ($org =~ /^MCAST-NET/);
	next if ($type =~ /early-registration/);
	next if ($org =~ /^EU-ZZ-\d/ && $rir eq "RIPE");

	next if ($type eq "rir");
	my @prefixes = split /,/, $prefixes;
	($date) = split /,/, $date;
	next unless ($date=~ /\d{8}/);
	my $date_frac = DateFraction($date);
print if ($date_frac >= 2002.6190 && $date_frac <= 2002.687);
#next unless ($date_frac > 2004.24863387978 && $date_frac < 2004.25409836066);

	foreach my $prefix (@prefixes) {
	    my ($front) = ($prefix =~ /^(\d+)/);
	    unless (defined $front) {
		die("Unable to parse front from prefix $prefix");
	    }
	    my $front_rir = $iana2rir{$front};
	    $rir = $front_rir if (defined $rir_target 
		&& $front_rir eq "various");
	    if (defined $rir_target && $rir ne $rir_target) {
		next;
	    }
	    next if ($front_rir ne $rir);
	    my ($addr, $len)  = split /\//, $prefix;
#print if ($len < 16);
#$len{$date}{$len}++;
	    
#my ($year, $mon, $day) = ($date =~ /(\d{4})(\d\d)(\d\d)/);
#my $time = $year + ($mon/12) + ($day/356);
#$date = $time;
#print STDERR if ($time >= 2004.335 && $time <= 2004.43);
#if ($prefixes =~ /114.0.0.0\/8/) { print; print "\t$key, $addr - $len -- $date\n";}
#print; print "\t$key, $addr - $len -- $date\n";

	    my $inet = inet_aton($addr);
	    my $masked = $len2mask{$len}&$inet;
	    if ($inet ne $masked) {
		my $masked_addr = inet_ntoa($masked);
		die("Masked $addr/$len should be $masked_addr:$_");
	    }
	    my $info = {
		"org" => $org, 
	#	"family" => $family,
	#	"addr" => $addr,
		"inet" => $inet,
		"len" => $len,
	#	"country" => $country,
	#	"type" => $type,
		"date" => $date_frac,
		"rir" => $rir,
		"size" => $len2size{$len}

		};
	    push @info, $info;
	$linenum++;
	}
#last if ($linenum > 10);
#last if (defined $last_rir && $rir ne $last_rir);
#$last_rir = $rir;
    }
=cut
    foreach my $date (sort keys %len) {
	print "$date\n";
	foreach my $len (sort {$a<=>$b} keys %{$len{$date}}) {
	    print "   $len $len{$date}{$len}\n";
	}
    }
=cut
}

sub DateFraction {
    my ($date) = @_;
    my ($year, $mon, $day) = ($date =~ /(\d{4})(\d\d)(\d\d)/);
    $day = "01" if ($day eq "00");
    $day = 28 if ((($year - 1980)%4 != 0) && $mon eq "02" && $day == 29);
    my ($s, $m, $h, $d, $m) = (0,0,0,1,0);
    my $start = Time::Local::timelocal($s,$m,$h,$d,$m,$year);
    my $current = Time::Local::timelocal($s,$m,$h,$day,$mon-1,$year);
    my $end = Time::Local::timelocal($s,$m,$h,$d,$m,$year+1);
    return $year + ($current-$start)/($end-$start);
}

sub CountRirs {
    print STDERR "I got here!\n";
    my @sorted = sort 
	{
	    my $a_inet = $a->{inet};
	    my $b_inet = $b->{inet};
	    if ($a_inet eq $b_inet) {
		return $a->{len} <=> $b->{len};
	    }
	    return $a_inet cmp $b_inet;
	} @info;
    my @stack;
    foreach my $info (@sorted) {
	my $start = $info->{inet};
	my $inet = unpack("N",$start) + $len2size{$info->{len}}-1;
	my $end = pack("N",$inet);
	while ($#stack >= 0 && $stack[0]{end} le $start) {
	    my $hash = shift @stack;
	    AddToSizes($hash);
	}

	my $size_first = $info->{size};
	my $size_last = $info->{size};
	if ($#stack >= 0) {
	    if ($stack[0]{date} <= $info->{date}) {
		$size_first = 0;
		$stack[0]{"size"}{"last"} -= $size_last;
	    } else {
		my $first_index = 0;
		while ($first_index <= $#stack 
		    && $stack[$first_index]{"size"}{"first"} <= 0) {
		    $first_index++;
		}
		    
		if ($first_index <= $#stack) {
		    $stack[$first_index]{"size"}{"first"} -= $size_first;
		}
		$stack[0]{"size"}{"last"} -= $size_last;
=cut
if ($stack[0]{"size"}{"first"} < 0) {
    print "$first_index\n";
    print "inet:",inet_ntoa($start),"/$info->{len} $size_first/$size_last\n";
    foreach my $index (0..$#stack) {

	    my $inet = $stack[$index]{"info"}{"inet"};
	    my $len = $stack[$index]{"info"}{"len"};

	    my $size = $stack[$index]{"info"}{"size"};
	    my $size_first = $stack[$index]{"size"}{"first"};
	    my $size_last = $stack[$index]{"size"}{"last"};

	    my $date = $stack[$index]{"info"}{"date"};
	    my $rir = $stack[$index]{"info"}{"rir"};
	    my $org = $stack[$index]{"info"}{"org"};

	    print   "     ",inet_ntoa($inet),"/",$len," ",
		    "masked:",inet_ntoa($inet&$len2mask{$len})," ",
		    "size:",$size," ",
		    "first:",$size_first," ",
		    "last:",$size_last," ",
		    "date:$date ",
		    "rir:$rir ",
		    "org:$org ",
		    "\n";
	    my $start = $stack[$index]{"start"};
	    my $end = $stack[$index]{"end"};
	    print   "        range:",inet_ntoa($start)
		," -> ",inet_ntoa($end),"\n";
    }
}
=cut
	    }
	}

	my $date = $info->{date};
	my $rir = $info->{rir};
	unshift @stack, {
	    "info" => $info,
	    "rir" => $rir,
	    "date" => $date,
	    "size"=>{
		"first" => $size_first,
		"last" => $size_last
		},
	    "start"=>$start,
	    "end"=>$end
	    };
    }
    while ($#stack >= 0) {
	my $hash = shift @stack;
	AddToSizes($hash);
    }

    @info = ();
}

sub AddToSizes{
    my ($hash) = @_;
    my $rir = $hash->{rir};
    my $date = $hash->{date};
    my $first = $hash->{"size"}{"first"};
    my $last = $hash->{"size"}{"last"};
    $sizes{$rir}{"first"}{$date}{$first}++;
    $sizes{$rir}{"last"}{$date}{$last}++;
}

sub GetTotals {
    my %totals;
    foreach my $rir (keys %sizes) {
	foreach my $type (keys %{$sizes{$rir}}) {
	    my $total = 0;
	    foreach my $date (sort {$a<=>$b} keys %{$sizes{$rir}{$type}}) {
		my $total_sub = 0;
		foreach my $size (sort {$a<=>$b} keys 
		    %{$sizes{$rir}{$type}{$date}}) {
		    $total_sub += $size*$sizes{$rir}{$type}{$date}{$size};
		}
		$total += $total_sub/$len2size{8};
		$totals{$rir}{$type}{$date} = $total;
	    }
	}
    }
    return \%totals;
}

sub GetRirs {
    my ($totals) = @_;
    my %last;
    foreach my $rir (keys %$totals) {
	if (defined $totals->{$rir}{"last"}) {
	    my @dates = sort {$a<=>$b} keys %{$totals->{$rir}{"last"}};
	    $last{$rir} = $totals->{$rir}{"last"}{$dates[$#dates]};
	}
    }
    return sort {$last{$a}<=>$last{$b}} keys %last;
}

sub PrintRirs {
    print STDERR "I got here 2!\n";
    my %info;
    my $totals = GetTotals();
    my @rir = GetRirs($totals);
    my $s = 0;
    foreach my $rir (@rir) {
	foreach my $type (keys %{$totals->{$rir}}) {
	    $type = "most specific" if ($type eq "last");
	    print "@ s$s legend \"$rir-$type\"\n";
	    $s++;
	    foreach my $date (sort {$a<=>$b} keys %{$totals->{$rir}{$type}}) {
		my $total = $totals->{$rir}{$type}{$date};
		my ($year, $mon, $day) = ($date =~ /(\d{4})(\d\d)(\d\d)/);
		my $time = $year + ($mon/12) + ($day/356);
		print "$date $total\n";
	    }
	}
    }
}

sub BuildLengthSize {
    my %len2size;
    foreach my $i (1..32) {
	$len2size{$i} = 1<<(32-$i);
    }
    return %len2size;
}

sub BuildLengthMask {
    my %len2size = @_;
    my %len2mask;
    my $full = unpack("N", inet_aton("255.255.255.255"));
    foreach my $i (1..32) {
	$len2mask{$i} = pack("N",$full<<(32-$i));
    }
    return %len2mask;
}
